package jalview.datamodel;

import static org.testng.Assert.assertEquals;

import jalview.io.DataSourceType;
import jalview.io.FileParse;
import jalview.io.HMMFile;

import java.io.IOException;
import java.net.MalformedURLException;

import org.testng.annotations.Test;

public class HiddenMarkovModelTest {

  HiddenMarkovModel hmm;

  @Test(priority = 0)
  public void testGetMatchEmissionProbabilities()
          throws MalformedURLException, IOException
  {
    HMMFile file = new HMMFile(new FileParse(
            "test/jalview/io/test_PKinase_hmm.txt", DataSourceType.FILE));
    hmm = file.getHMM();

    double[] actual = new double[4];
    actual[0] = hmm.getMatchEmissionProbability(0, 'R');
    actual[1] = hmm.getMatchEmissionProbability(19, 'W');
    actual[2] = hmm.getMatchEmissionProbability(160, 'G');
    actual[3] = hmm.getMatchEmissionProbability(475, 'A');

    double[] expected = new double[4];
    expected[0] = 0.02537400637;
    expected[1] = 0.00588228492;
    expected[2] = 0;
    expected[3] = 0.04995163708;

    for (int i = 0; i < 4; i++)
    {
      assertEquals(actual[i], expected[i], 0.001d);
    }
  }
  
  @Test(priority = 1)
  public void testGetInsertEmissionProbabilities()
  {
    double[] actual = new double[4];
    actual[0] = hmm.getInsertEmissionProbability(2, 'A');
    actual[1] = hmm.getInsertEmissionProbability(5, 'T');
    actual[2] = hmm.getInsertEmissionProbability(161, 'K');
    actual[3] = hmm.getInsertEmissionProbability(472, 'L');

    double[] expected = new double[4];
    expected[0] = 0.06841384927;
    expected[1] = 0.06233763141;
    expected[2] = 0;
    expected[3] = 0.06764038926;

    for (int i = 0; i < 4; i++)
    {
      assertEquals(actual[i], expected[i], 0.001d);
    }
  }

  @Test(priority = 1)
  public void testGetStateTransitionProbabilities()
  {
    double[] actual = new double[4];
    actual[0] = hmm.getStateTransitionProbability(475, hmm.MATCHTODELETE);
    actual[1] = hmm.getStateTransitionProbability(8, hmm.MATCHTOINSERT);
    actual[2] = hmm.getStateTransitionProbability(80, hmm.INSERTTOINSERT);
    actual[3] = hmm.getStateTransitionProbability(475, hmm.DELETETOMATCH);

    double[] expected = new double[4];
    expected[0] = Double.NEGATIVE_INFINITY;
    expected[1] = 0.00662894243;
    expected[2] = 0.46183388908;
    expected[3] = 1;

    for (int i = 0; i < 4; i++)
    {
      assertEquals(actual[i], expected[i], 0.001d);
    }
  }
  
  @Test(priority = 1)
  public void testGetConsensusAtAlignColumn()
  {
    char[] cons = new char[4];
    cons[0] = hmm.getConsensusAtAlignColumn(10);
    cons[1] = hmm.getConsensusAtAlignColumn(50);
    hmm.setConsensusResidueStatus(false);
    cons[2] = hmm.getConsensusAtAlignColumn(100);
    cons[3] = hmm.getConsensusAtAlignColumn(400);
    
    char[] expected = new char[4];
    expected[0] = 's';
    expected[1] = 'k';
    expected[2] = 'l';
    expected[3] = 'k';
    
    for (int i = 0; i < 4; i++)
    {
      assertEquals(cons[i], expected[i]);
    }
    
  }

  @Test(priority = 1)
  public void testGetConsensusSequence()
  {
    SequenceI seq = hmm.getConsensusSequence();
    String subStr = seq.getSequenceAsString().substring(0, 10);
    assertEquals(subStr, "YELLEKLGSG");
    subStr = seq.getSequenceAsString().substring(150, 161);
    assertEquals(subStr, "-DLLK------");
  }
}
